import traceback

from django.db import transaction, connection

from dm.utils.logging import logger


class BaseDataMigrationTask:
    def __init__(self, **kwargs):
        self.version = None
        self.run_in_public_schema = True
        self.run_in_custom_schema = True

    def _run(self):
        raise NotImplementedError

    @classmethod
    def get_current_schema_name(cls):
        return getattr(connection, "schema_name", None)

    def run(self, **kwargs):
        current_schema_name = self.get_current_schema_name()
        if current_schema_name is not None:
            if not self.run_in_public_schema and current_schema_name == "public":
                return (
                    True,
                    "Skipping running the data migration task in public schema.",
                )
            if not self.run_in_custom_schema and current_schema_name != "public":
                return (
                    True,
                    "Skipping running the data migration task in custom schema.",
                )

        if self.version is None:
            raise Exception(
                "Version not defined. Skipping running the data migration task."
            )

        prior_atomic = connection.in_atomic_block
        if not prior_atomic:
            transaction.set_autocommit(False)
        try:
            self._run()
        except Exception as e:
            logger.info(
                f"Exception occurred while running the Data Migration task - {str(e)}"
            )
            traceback.print_exc()
            if not prior_atomic:
                transaction.rollback()
                transaction.set_autocommit(True)
            return False, str(e)
        else:
            if not prior_atomic:
                transaction.commit()
                logger.info("Transaction committed!")
                transaction.set_autocommit(True)

        return True, None



from django.contrib import admin
from dm.models import DataMigrationExecution


@admin.register(DataMigrationExecution)
class DataMigrationExecutionAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "task",
        "execution_version",
        "status",
        "created_on",
        "updated_on",
    )
    ordering = ["-updated_on"]
    list_per_page = 25
    list_filter = ("status",)
    search_fields = ("id", "execution_version", "task")



import logging
from json_log_formatter import JSONFormatter

logger = logging.getLogger()


class StandardJSONLogFormatter(JSONFormatter):
    def json_record(self, message, extra, record):
        additional_info = {
            "name": record.name,
            "level": record.levelname,
            "file": record.filename,
            "exc_info": record.exc_info,
            "thread": record.thread,
        }
        extra = {**extra, **additional_info}
        return super(StandardJSONLogFormatter, self).json_record(message, extra, record)



import importlib
import time

from dm.managers.dm.core import DataMigrationManager
from django.conf import settings

from dm.utils.logging import logger


class DataMigrationRunner:
    @classmethod
    def _get_base_list_of_task_classes(cls):
        base_task_classes = []
        try:
            base_task_classes = settings.DATA_MIGRATION_REGISTRY
        except Exception as e:
            logger.warning(
                f"Exception occurred while trying to get base list of data migration tasks - {str(e)}"
            )
        return base_task_classes

    @classmethod
    def _get_retry_config(cls):
        total_retries, retry_delay_in_seconds = 5, 5
        try:
            total_retries = settings.DATA_MIGRATION_TOTAL_RETRIES
            retry_delay_in_seconds = settings.DATA_MIGRATION_RETRY_DELAY_IN_SECONDS
        except Exception:
            pass
        return total_retries, retry_delay_in_seconds

    def get_all_dm_tasks_classes(self):
        return self._get_base_list_of_task_classes()

    def get_retry_config(self):
        return self._get_retry_config()

    @classmethod
    def import_and_get_class(cls, task_class_string):
        full_class_path = task_class_string.split(".")
        class_name = full_class_path[-1]
        class_path = ".".join(full_class_path[:-1])
        module_containing_class = importlib.import_module(name=class_path)
        task_class = getattr(module_containing_class, class_name)
        return task_class

    def _run(self):
        all_task_classes_strings, failure_recorded = (
            self.get_all_dm_tasks_classes(),
            False,
        )
        for task_class_string in all_task_classes_strings:
            task_class = self.import_and_get_class(task_class_string=task_class_string)
            dm_manager = DataMigrationManager(task_class=task_class)
            success = dm_manager.run()
            if success is False:
                failure_recorded = True
        return failure_recorded

    def run(self):
        total_retries, retry_delay_in_seconds = self.get_retry_config()
        attempt = 1
        while attempt <= total_retries:
            time.sleep(retry_delay_in_seconds * (attempt - 1))
            logger.info(f"Running Data Migration tasks - attempt {attempt}")
            failure = self._run()
            if not failure:
                break
            logger.info(
                f"One or more Data Migration Tasks failed for attempt {attempt}. "
                f"Attempting to retrying Data Migration tasks..."
            )
            attempt += 1

        if attempt > total_retries:
            logger.error(
                f"Data Migration tasks failed after {total_retries} attempts. "
                f"Please check the logs for more details."
            )
            return False

        logger.info(
            f"Data Migration tasks completed successfully in {attempt} attempt(s)."
        )
        return True



import traceback

from dm.constants import DataMigrationStatus
from dm.models import DataMigrationExecution
from dm.utils.logging import logger


class DataMigrationManager:
    def __init__(self, **kwargs):
        self.dm_exec_obj = kwargs.get("dm_exec_obj")
        self.task_class = kwargs.get("task_class")
        self.task_class_name = self.task_class.__name__ if self.task_class else None

    def _get_existing_dm_exec_obj(self, execution_version):
        return DataMigrationExecution.objects.filter(
            task=self.task_class_name, execution_version=execution_version
        ).first()

    def init_dm(self, execution_version):
        existing_dm_exec_obj = self._get_existing_dm_exec_obj(
            execution_version=execution_version
        )
        self.dm_exec_obj = existing_dm_exec_obj
        if existing_dm_exec_obj is None:
            self.dm_exec_obj = DataMigrationExecution.objects.create(
                task=self.task_class_name, execution_version=execution_version
            )
            return self.dm_exec_obj, "new_obj"

        if existing_dm_exec_obj.status == DataMigrationStatus.COMPLETED.value:
            return None, "already_completed"

        if existing_dm_exec_obj.status == DataMigrationStatus.FAILED.value:
            return self.dm_exec_obj, DataMigrationStatus.FAILED.value

        return self.dm_exec_obj, "existing_obj"

    def _update_dm_status(self, dm_status):
        self.dm_exec_obj.status = dm_status
        self.dm_exec_obj.save()
        return self.dm_exec_obj

    def mark_dm_as_failed(self):
        return self._update_dm_status(dm_status=DataMigrationStatus.FAILED.value)

    def mark_dm_as_completed(self):
        return self._update_dm_status(dm_status=DataMigrationStatus.COMPLETED.value)

    def run(self):
        if not self.task_class:
            logger.info("No Data Migration task class specified")
            return None

        task_obj = self.task_class()
        try:
            self.dm_exec_obj, message = self.init_dm(execution_version=task_obj.version)
        except Exception as e:
            logger.info(
                f"Exception occurred while initializing Data Migration task "
                f"{self.task_class_name} v{task_obj.version} - {str(e)}"
            )
            traceback.print_exc()
            return False

        if self.dm_exec_obj is None and message:
            if message != "already_completed":
                logger.info(
                    f"Cannot execute task {self.task_class_name} v{task_obj.version}. Reason: {message}"
                )
            return None

        logger.info(f"Running {self.dm_exec_obj.__str__()}")
        try:
            status, error_message = task_obj.run()
        except Exception as e:
            logger.info(
                f"Exception occurred while running {self.dm_exec_obj.__str__()}: {str(e)}"
            )
            traceback.print_exc()
            self.mark_dm_as_failed()
            return False

        if status is False:
            logger.info(
                f"Data Migration task {self.dm_exec_obj.__str__()} failed: {error_message}"
            )
            self.mark_dm_as_failed()
            return False

        self.mark_dm_as_completed()
        return True



from django.contrib import admin
from dmf_test_app.models import FeatureSwitch


@admin.register(FeatureSwitch)
class FeatureSwitchAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "name",
        "description",
        "active",
        "created_on",
        "updated_on",
    )
    ordering = ["-updated_on"]
    list_per_page = 25
    list_filter = ("active",)
    search_fields = ("id", "name")



from enum import Enum


class DataMigrationStatus(Enum):
    IN_PROGRESS = "IN_PROGRESS"
    FAILED = "FAILED"
    COMPLETED = "COMPLETED"

    @classmethod
    def choices(cls):
        return [(i.name, i.value) for i in cls]



# Generated by Django 5.2.1 on 2025-06-21 15:01

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="FeatureSwitch",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=500, unique=True)),
                ("description", models.TextField(blank=True, null=True)),
                ("active", models.BooleanField(default=True)),
            ],
        ),
    ]
